'''
Creates an xml file of a pattern of notes and durations in a selection of specific keys.
Can be extended to multiple patterns by creating a lists of lists for each pattern and adding extra loops
'''

from music21 import *  # get the music21 library....

# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Set the parameters for the exercises
keyList = "C D- D E- E F G- G A- A B- B"  # a set of keys to cover...
pitchPattern = [[1, 4, 3, 2, 6, 5, 9, 8],
                [3, 1, 4, 2, 5, 3, 1, 8]
                ]  # scale degrees for each exercise (here we have 2 set of pitches for each rhythmic pattern)
durPattern = [1, 0.75, 0.25, 1, 0.5, 0.5, 2, 2]  # the rhythmic pattern for the exercise
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++

k = keyList.split(" ")  # create a list for the keys
s = stream.Stream()  # create a music stream to hold the music
s.insert(0, meter.TimeSignature('4/4'))  # add a time signature at the start

for currKey in k:  # for each key
    thisKey = key.Key(currKey)  # create a key object...
    exercise = stream.Stream()  # create a stream to hold this exercise
    exercise.insert(0, thisKey)  # add the key signature
    for pat in pitchPattern: # for each pattern of pitches
        for scaleDeg, noteDuration in zip(pat, durPattern):  # for each pitch and duration in the pattern...
            thisNote = note.Note()  # create a note
            thisNote.pitch = thisKey.pitchFromDegree(scaleDeg)  # set the pitch depending on the key
            thisNote.octave += scaleDeg // 8  # adjust for octave (if necessary)
            thisNote.quarterLength = noteDuration  # set the note duration
            exercise.append(thisNote)  # append the note to this pattern

    s.append(exercise)  # add this pattern to the overall stream

# tidy up...
flatStream = s.flatten()  # flatten the stream (to avoid recursing over each pattern)
flatStream.removeByClass('TrebleClef')  # get rid of surplus clefs... [note sure why these appear!]
flatStream.show()  # create the xml file
